from IPython import embed

import collections
import csv

patterns = {
    "Visit": set(['search',
                  'reach',
                  'reacharea',
                  'reachobject',
                  'reachobjectreach',
                  'reachsearch',
                  'visit',
                  'visitarea',
                  'locate',
                  'cover',
                  'explore',
                  'meet',
                  'searcharea', ]),
    "Sequence Visit ": set(['seqvisit',
                            'seqvist', 'sequence']),
    "Strict Visit": set(['strictorder',
                         'strictorderedvisit',
                         'strictsequence',
                         'strictvisit', ]),
    "Fair Visit": set(),
    "Patrolling": set(['patrol',
                       'surveillance',
                       'patrolling', ]),
    "Sequenced Patrolling": set('seqpatrol'),
    "Strict Patrolling": set(['strictpatrol', ]),
    "Fair Patrolling": set(['fairpatrol', ]),
    "Past Avoidance": set(['pastavoid', ]),
    "Global Avoidance": set(['globalavoidance',
                             'globalavoid',
                             'avoidglobal',
                             'visitavoid',
                             'gavoid',
                             'avoid',
                             'gavoidance', ]),
    "Future Avoidance": set(['futureavoid', ]),
    "Lower Restricted Avoidance ": set(['loweravoid', ]),
    "Exact Restricted Avoidance": set(),
    "Upper Restricted Avoidance ": set(['avoidupper',
                                        'upperavoid',
                                        'upperbound',
                                        'upperrestrictavoid', ]),
    "Instant Reaction": set(['ireaction',
                             'reaction',
                             'trigger', ]),
    "Delayed Reaction": set(['dreaction', ]),
    "Wait": set(['wait', 'stop', ]),
}

requirements_terms_str = """follow,trajectory,formation
moveobject,follow
follow,trajectory,reach
patrol
follow,reach,reaction,upperbound
restrict,ireaction,upperbound
gavoidance,sequence,visit
reacharea
visit
reach
searcharea
reacharea
visit
avoid,collision
visitarea
visit,avoidcollision
visit,avoidcollision
strictorderedvisit,gavoid
strictorderedvisit
reach,strictorder
reachobject,reacharea,trigger
strictorderedvisit
strictorderedvisit,wait
strictorderedvisit
reach,trajectory
ireaction,reach
cover,upperrestrictavoid
reach
gavoid,reach
search
search,gavoid
reach
search,globalavoidance
timebound,reach
reach,strictorder,spacebound
reachobject
reach
follow,avoidglobal
reach
trajectory,globalavoid
formation
formation
surveillance,wait,reach
reach
reach,avoidupper,timebound
reach
reach,gavoid
reach,gavoid
reach,gavoid
reach,gavoid
reach
search,reach,gavoid,trajectory
reacharea
trajectory
formation
trajectory,gavoid
formation
search
gavoid,trajectory
formation
cover
seqvist
surveillance
search,ireaction
gavoid,trajectory
reach
reach,ireaction
search,gavoid,ireaction
gavoid,spacebound,ireaction
reach,ireaction
reach,gavoid
trajectory
trajectory
reach
reach
gavoid,search
gavoid,spacebound
cover
reach
formation
trajectory
cover
cover
patrol
explore
trajectory
gavoid
strictsequence
cover
patrol,gavoid,timebound
sequence
cover
trajectory,reach
gavoid
sequence,visit
trajectory
explore,formation
cover,search,gavoid
surveillance
cover
cover,loweravoid
cover,loweravoid
trajectory,timebound
trajectory,timebound
visit
sequence
futureavoid,visit,ireaction
pastavoid,visit,gavoid
search
patrol
pastavoid,visit
dreaction,search
patrol,futureavoid
futureavoid,patrol
visit,futureavoid
search
search
cover
patrol,futureavoid
upperavoid,timebound
patrolling, futureavoid
ireaction,patrol
patrol,ireaction,reach
patrol
gavoid,ireaction
gavoid,ireaction
gavoid,reach
gavoid,timebound,ireaction
gavoid,timebound,ireaction,dreaction
gavoid,timebound,ireaction,dreaction
patrol
cover,patrol
gavoid,reach
reachobject
reach
search,gavoid, follow
search,meet
reach
visit
seqvisit
visit
reach
gavoid
reach
reach,gavoid
reach,gavoid
reach,gavoid
reach,gavoid
follow
stop
reach
patrol
gavoid,trajectory,timebound
reach,timebound
loweravoid,cover
gavoid,patrol
search
cover
surveillance
patrol,timebound
surveillance
visit
visit,loweravoid,gavoid
cover
formation
spacebound,formation
spacebound
cover
strictvisit
seqvisit,gavoid
visit
strictvisit
gavoid
patrol
strictpatrol,ireaction
reach,gavoid
patrol
ireaction
reach
reach
reach
reach
reach
reach
patrol,gavoid,ireaction
patrol,gavoid
patrol,gavoid
fairpatrol
reach,response,gavoid,ireaction
gavoid,sequence
ireaction,gavoid
reach
gavoid,strictvisit
timebound
sequence
sequence
sequence,reach,ireaction
sequence,reach,ireaction
spacebound
spacebound
locate
balance
balance
reach,gavoid
reachobject
search,dreaction,sequence,wait
spacebound
visit,gavoid
patrol
seqvisit
ireaction,seqvisit
patrol
ireaction,patrol
ireaction,patrol
strictorderedvisit
strictorderedvisit, ireaction
visit
visit,gavoid
visit,gavoid
visit
visit,ireaction
gavoid
pastavoid
gavoid
gavoid
gavoid
gavoid
patrol
visit,gavoid
futureavoid
patrol
patrol
ireaction,dreaction
visit
surveillance
ireaction,visit
patrol
wait
seqpatrol
wait,ireaction
seqpatrol
seqvisit
visit
visit,seqvisit
visit,ireaction
visit,ireaction
patrol,ireaction
visit,gavoid
visit,gavoid
visit,gavoid
visit,seqvisit,ireaction,futureavoid,pastavoid
visit,gavoid
surveillance,ireaction,visit
surveillance,ireaction,visit
visit,patrol,ireaction
patrol
pastavoid,visit
futureavoid,wait"""



# embed()
requirements_terms=[]
for line in requirements_terms_str.split('\n'):
	requirements_terms.append(set(line.split(',')))


print requirements_terms


all_requirement_terms = set.union(*requirements_terms)

all_pattern_terms = set.union(*patterns.values())

occurences = {key: 0 for key in patterns.keys()}

not_supported_terms = all_requirement_terms - all_pattern_terms

occurences_count = 0
for pattname, pattkeywords in patterns.items():
    for reqterms in requirements_terms:
        if any(x in pattkeywords for x in reqterms):
            occurences[pattname] += 1
            occurences_count += 1

# count the requirements that have at least a concern not covered by patterns
not_covered_reqs = []
for reqterms in requirements_terms:
    if not reqterms.issubset(all_pattern_terms):
        not_covered_reqs.append(reqterms)


multiple_occurences_patterns = {key: 0 for key in patterns.keys()}

# find out combinations of pattern occurences
pattern_occurences_all = []
for reqterms in requirements_terms:
    pattern_occurences = []
    for pattname, pattkeywords in patterns.items():
        if any(x in pattkeywords for x in reqterms):
            pattern_occurences.append(pattname)
    if pattern_occurences:
        pattern_occurences_all.append(frozenset(pattern_occurences))

print 'Occurences of patterns:'
for i in collections.Counter(occurences).most_common():
    print i
print

print "Occurences count", sum(occurences.values())

print
# find out frequencies of combinations
counter = collections.Counter(pattern_occurences_all)
print 'Frequencies of combinations:'
for i in counter.most_common():
    print i
print

print "Frequency of usage of each pattern:"
for k, v in occurences.items():
    print k, v, "(" + str((v * 100.0) / occurences_count) + "%)"

print "Not covered requirements", len(not_covered_reqs)
print not_covered_reqs
print "Requirements that have at least a concern not covered by patterns", len(not_covered_reqs), "(" + str((len(not_covered_reqs) * 100.0) / len(requirements_terms)) + "%)"


print 'Total reqs',len(requirements_terms)


print 'Terms not covered by patterns ',not_supported_terms

# outfile  = open('papers-reqs-considered.csv', "wb")
# print 'writing papers-reqs-considered.csv'
# writer = csv.writer(outfile, delimiter=',', quotechar='"', quoting=csv.QUOTE_ALL)


# with open('papers.csv') as csvfile:
#     readCSV = csv.reader(csvfile, delimiter=',')
#     for row in readCSV:
#         try:
#             writer.writerow([row[0], row[2], row[3], row[10],row[9]])
#         except IndexError:
#             continue
